#!/usr/bin/php
<?php

$dsn = "mysql:dbname=psa;hostname=localhost";
$user = "admin";
$pass = trim(file_get_contents("/etc/psa/.psa.shadow"));
$psaCertificateUtil = '/usr/local/psa/bin/certificate';
$nginxManager = '/usr/local/psa/admin/bin/nginxmng';

function getCertificateRow($conn, $name){
    $sql = 'select id, csr, pvt_key, cert, cert_file, ca_cert, name from certificates where name like :name limit 1';
    $q  = $conn->prepare($sql);
    $q->execute(array(':name' => $name));
    return $q->fetch();
}

function getAllCertificateRows($conn){
    $sql = 'select id, csr, pvt_key, cert, cert_file, ca_cert, name from certificates';
    $q  = $conn->prepare($sql);
    $q->execute();
    return $q->fetchAll();
}


function createTempFile($content) {
    $file = tempnam(sys_get_temp_dir(), 'plesk_cert');
    return file_put_contents($file, $content) ? $file : false;
}

function createTempCertificateFiles($keyParts) {
    $keyPartsWithFiles = array();
    foreach ($keyParts as $name => &$part) {
        if (isset($part['content']) && !empty($part['content'])) {
            $part['file'] = createTempFile(urldecode($part['content']));
            $keyPartsWithFiles[$name] = $part;
        }
    }
    return $keyPartsWithFiles;
}

function deleteTempCertificateFiles($keyPartsWithFiles) {
    $answer = true;
    foreach ($keyPartsWithFiles as $part) {
        $answer = $answer && unlink($part['file']);
    }
    return $answer;
}

function getDomainNameFromCertificate($conn, $certificate) {
    $sql = 'select domains.name from certificates join Repository on (Repository.component_id=certificates.id) join domains on (domains.cert_rep_id = Repository.rep_id) where certificates.id = :id ';
    $q  = $conn->prepare($sql);
    $q->execute(array(':id' => $certificate['id']));
    $res = $q->fetch();
    return $res['name'];
}


function updateAdminCertificate($psaCertificateUtil, $certificate, $keyPartsWithFiles){
    $command = '';
    if (empty($certificate['name'])) {
        exit("Certifacate id:{$certificate['id']} name is empty.");
    } else {
        $command .= "{$psaCertificateUtil} --update '{$certificate['name']}' -admin";
    }
    $command .= addFileOptions($keyPartsWithFiles);
    $result = `$command`;
    echo "{$command}{$result}\n";
}

function updateDomainCertificate($psaCertificateUtil, $certificate, $keyPartsWithFiles, $domainName){
    $command = '';
    if (empty($certificate['name'])) {
        exit("Certifacate id:{$certificate['id']} name is empty.");
    } else {
        $command .= "{$psaCertificateUtil} --update '{$certificate['name']}'";
    }
    if (empty($domainName)) {
        exit("Certifacate have not domain.");
    } else {
        $command .= " -domain '{$domainName}'";
    }
    $command .= addFileOptions($keyPartsWithFiles);
    $result = `$command`;
    echo "{$command}{$result}\n";
}

function addFileOptions($keyPartsWithFiles) {
    $fileOptions = '';
    if (!empty($keyPartsWithFiles['csr']['file'])) {
        $fileOptions .= " -csr-file '{$keyPartsWithFiles['csr']['file']}'";
    }
    if (!empty($keyPartsWithFiles['key']['file'])) {
        $fileOptions .= " -key-file '{$keyPartsWithFiles['key']['file']}'";
    }
    if (!empty($keyPartsWithFiles['cert']['file'])) {
        $fileOptions .= " -cert-file '{$keyPartsWithFiles['cert']['file']}'";
    }
    if (!empty($keyPartsWithFiles['ca_cert']['file'])) {
        $fileOptions .= " -cacert-file '{$keyPartsWithFiles['ca_cert']['file']}'";
    }
    return $fileOptions;
}

function reloadCertificate($psaCertificateUtil, $conn, $certificate) {
    $keyParts = array();
    if (isset($certificate['csr']) && !empty($certificate['csr'])) {
        $keyParts['csr']['content'] = $certificate['csr'];
    }
    if (isset($certificate['pvt_key']) && !empty($certificate['pvt_key'])) {
        $keyParts['key']['content'] = $certificate['pvt_key'];
    }
    if (isset($certificate['cert']) && !empty($certificate['cert'])) {
        $keyParts['cert']['content'] = $certificate['cert'];
    }
    if (isset($certificate['ca_cert']) && !empty($certificate['ca_cert'])) {
        $keyParts['ca_cert']['content'] = $certificate['ca_cert'];
    }

    $keyPartsWithFiles = createTempCertificateFiles($keyParts);
    $domainName = getDomainNameFromCertificate($conn, $certificate);

    if (isset($domainName) && !empty($domainName)) {
        updateDomainCertificate($psaCertificateUtil, $certificate, $keyPartsWithFiles, $domainName);
    } else {
        updateAdminCertificate($psaCertificateUtil,$certificate, $keyPartsWithFiles);
    }
    deleteTempCertificateFiles($keyPartsWithFiles);
}

function rewriteNginxConfig($nginxManager) {
    $command = "$nginxManager --disable";
    $result = `$command`;
    echo "{$command}{$result}\n";
    $command = "$nginxManager --enable";
    $result = `$command`;
    echo "{$command}{$result}\n";
}

if (in_array($argv[1], array('--all', '-a'))) {
    try {
        $conn = new PDO($dsn, $user, $pass);
        foreach (getAllCertificateRows($conn) as $certificate) {
            reloadCertificate($psaCertificateUtil, $conn, $certificate);
        }
        rewriteNginxConfig($nginxManager);
    } catch (Excaption $e) {
        echo $e->getMessage();
    }
} else if (in_array($argv[1], array('--single', '-s'))) {
    try {
        $conn = new PDO($dsn, $user, $pass);
        $certificate = getCertificateRow($conn, $argv[2]);
        reloadCertificate($psaCertificateUtil, $conn, $certificate);
        rewriteNginxConfig($nginxManager);
    } catch (Excaption $e) {
        echo $e->getMessage();
    }
} else {
    echo "command:\n";
    echo "\n";
    echo "\t--all, -a - reload all SSL sertificates\n";
    echo "\texample:\n";
    echo "\t\t> {$argv[0]} --all \n";
    echo "command:\n";
    echo "\n";
    echo "\t--single, -s - reload one SSL sertificate\n";
    echo "\tsecond argument is name SSL sertificate (required)\n";
    echo "\texample:\n";
    echo "\t\t> {$argv[0]} --single 'default certificate'\n";
}
